
import java.lang.reflect.Field;
import java.time.LocalDate;
import java.util.*;

public class Checkbook {
    // Checkbook is a list of Transactions

    private List<Transaction> checkbook;

    public Checkbook(Transaction ... t) {
        this.checkbook = new ArrayList<>(Arrays.asList(t));
    }

    public Checkbook(List<Transaction> checkbook) {
        this.checkbook = new ArrayList<>(checkbook);
    }

    public Checkbook(double cashAmount) {
        this();
        checkbook.add(new Transaction("Default", new Payee(), TransactionType.DEBIT, cashAmount ));
    }

    public Checkbook(Payee owner, double cashAmount) {
        this();
        checkbook.add(new Transaction("Transaction", owner, TransactionType.DEBIT, cashAmount));
    }

    //blind us with your Java eloquence....

    public int size() {
        return this.checkbook.size();
    }

    public boolean isEmpty() {
        return this.checkbook.isEmpty();
    }

    public boolean contains(Transaction transaction) {
        return this.checkbook.contains(transaction);
    }

    public boolean add(Transaction transaction) {
        return this.checkbook.add(transaction);
    }

    public boolean remove(Transaction transaction) {
        return this.checkbook.remove(transaction);
    }

    public Iterator<Transaction> iterator() {
        return this.checkbook.iterator();
    }

    public Transaction[] toArray() {
        return checkbook.toArray(new Transaction[checkbook.size()]);
    }

    public Transaction[] toArray(Transaction[] a) {
        return this.checkbook.toArray(a);
    }

    public Transaction[] toArrayFromList(List<Transaction> list) {
        return list.toArray(new Transaction[list.size()]);
    }

    public double checkbookValue() {
        double sum = 0.0;
        for (Transaction t: this.checkbook) {
            sum += t.getAmount();
        }
        return sum;
    }

    public ArrayList<Transaction> getCheckbook() {
        return checkbook;
    }

    public Transaction[] getSpecificTransactions(String fieldName, Object value) {

        List<Transaction> arr = new ArrayList<>();
        for (Transaction ts : this.checkbook) {
            Object valueToCheck = getObject(fieldName, ts);
            if (valueToCheck.equals(value)) {
                arr.add(ts);
            }
        }
        return this.toArrayFromList(arr);
    }

    private Object getObject(String fieldName, Transaction ts) {
        try {
            Class<Transaction> t = Transaction.class;
            Field f = t.getDeclaredField(fieldName);
            f.setAccessible(true);
            return f.get(ts);
        } catch (NoSuchFieldException | IllegalAccessException noField) {
            noField.printStackTrace();
        }

        return null;
    }


    public boolean containsAll(Collection<Transaction> c) {
        return checkbook.containsAll(c);
    }

    boolean addAll(Collection<Transaction> c) {
        return checkbook.addAll(c);
    }

    boolean removeAll(Collection<Transaction> c) {
        return checkbook.removeAll(c);
    }

    void clear() {
        checkbook.clear();
    }

    private Transaction[] getAllTransactionByType(TransactionType tt) {
        ArrayList<Transaction> arr = new ArrayList<>();
        for (Transaction t: this.checkbook) {
            if (t.getType().equals(tt)) {
                arr.add(t);
            }
        }
        return this.toArrayFromList(arr);
    }


    public Transaction[] getAllDebitTransactions() {
        return getAllTransactionByType(TransactionType.DEBIT);
    }

    public Transaction[] getAllCreditTransactions() {
        return getAllTransactionByType(TransactionType.CREDIT);
    }
}
