import org.junit.Assert;
import org.junit.Test;

import java.time.LocalDate;
import java.util.NoSuchElementException;

import static org.junit.Assert.*;

public class CheckbookTest {
    private Checkbook checkbook, checkbookWithDefault, checkbookWithOwner;
    private Payee p1, p2, p3;
    private Transaction t1, t2, t3;
    private final double cashAmount = 500.00;
    private final double t2Amount = 100.00;
    private final double t3Amount = 50.00;
    private final double DELTA = 0.0;

    @org.junit.Before
    public void setUp(){

        p1 = new Payee("Bear", "Luna", "luba@gmail.com");
        p2 = new Payee("Bear", "Rocco", "roba@gmail.com");
        p3 = new Payee("Fish", "Salmon", "salmonBear@gmail.com");

        t1 = new Transaction("ZC Fund", p1, TransactionType.CREDIT, cashAmount);
        t2 = new Transaction("Allowance", p3, TransactionType.DEBIT, t2Amount);
        t3 = new Transaction("Food", p2, TransactionType.CREDIT, t3Amount);

        checkbook = new Checkbook(t1, t2, t3);
        checkbookWithDefault = new Checkbook(t2Amount);
        checkbookWithOwner = new Checkbook(p1, t3Amount);

    }

    @Test
    public void testSizeVarConstructor() {
        int expectSizeInitial = 3;
        assertEquals(expectSizeInitial, checkbook.size());

        expectSizeInitial = 1;
        assertEquals(expectSizeInitial, checkbookWithDefault.size());
        assertEquals(expectSizeInitial, checkbookWithOwner.size());
    }

    @Test
    public void testSizeRemove() {
        int expectedSizeRemove = 2;

        checkbook.remove(t1);
        assertEquals(expectedSizeRemove, checkbook.size());
    }

    @Test
    public void testSizeCashAmountConstructor() {
        Transaction t = new Transaction("new", p2, TransactionType.CREDIT, 2.00);
        int expectedSizeAdd = 2;

        checkbookWithDefault.add(t);
        checkbookWithOwner.add(t);

        assertEquals(expectedSizeAdd, checkbookWithDefault.size());
        assertEquals(expectedSizeAdd,checkbookWithOwner.size());

        expectedSizeAdd = 4;
        checkbook.add(t);

        assertEquals(expectedSizeAdd, checkbook.size());
    }

    @Test
    public void testIsEmptyTrueDefault() {
        Checkbook emptyCB = new Checkbook();
        assertTrue(emptyCB.isEmpty());
    }

    @Test
    public void testIsEmptyFalse() {
        assertFalse(checkbook.isEmpty());
        assertFalse(checkbookWithOwner.isEmpty());
        assertFalse(checkbookWithDefault.isEmpty());
    }

    @Test
    public void testIsEmptyRemove() {
        checkbook.remove(t1);
        checkbook.remove(t2);
        checkbook.remove(t3);

        assertTrue(checkbook.isEmpty());
    }

    @Test
    public void testContainsTrue() {
        assertTrue(checkbook.contains(t1));

        checkbookWithDefault.add(t1);
        assertTrue(checkbookWithDefault.contains(t1));

        checkbookWithOwner.add(t3);
        assertTrue(checkbookWithOwner.contains(t3));
    }

    @Test
    public void testContainsFalse() {
        checkbook.remove(t1);
        assertFalse(checkbook.contains(t1));

        assertFalse(checkbookWithDefault.contains(t1));
        assertFalse(checkbookWithDefault.contains(t3));
    }

    @Test
    public void testContains() {

        checkbookWithOwner.add(t1);
        assertTrue(checkbookWithOwner.contains(t1));
        assertFalse(checkbookWithOwner.contains(t2));
    }

    @Test
    public void testIteratorFalse() {
        Checkbook cb = new Checkbook();
        assertFalse(cb.iterator().hasNext());

        assertTrue(checkbook.iterator().hasNext());
        assertTrue(checkbookWithDefault.iterator().hasNext());
        assertTrue(checkbookWithOwner.iterator().hasNext());
    }

    @Test(expected = NoSuchElementException.class)
    public void testIteratorException() {
        Checkbook cb = new Checkbook();
        cb.iterator().next();
    }

    @Test
    public void testIteratorTrueFalseFalse() {
        assertTrue(checkbook.iterator().hasNext());
        checkbook.remove(t1);
        checkbook.remove(t2);
        checkbook.remove(t3);

        assertFalse(checkbook.iterator().hasNext());
        assertFalse(checkbook.iterator().hasNext());
    }

    @Test
    public void testToArray() {
        Transaction[] arrEx = {t1, t2, t3};
        assertEquals(arrEx,checkbook.toArray());
    }

    @Test
    public void testToArrayFail() {
        Transaction[] arrUnEx = {};
        assertFalse(arrUnEx.equals(checkbook.toArray()));
    }

    @Test
    public void testToArrayOtherConstructors() {
        Transaction t1 = checkbookWithDefault.getCheckbook().get(0);
        Transaction[] arr = {t1};

        assertEquals(arr, checkbookWithDefault.toArray());

        t1 = checkbookWithOwner.getCheckbook().get(0);
        Transaction[] arr2 = {t1};
        assertEquals(arr2, checkbookWithOwner.toArray());
    }

    @Test
    public void testToArrayV2() {
        Transaction[] arrEx = {t1, t2, t3};

        assertEquals(arrEx,checkbook.toArray(arrEx));
    }

    @Test
    public void testToArrayFailV2() {
        Transaction[] arrUnEx = {t1, t2};
        Transaction[] arr = {t1};
        assertFalse(arr.equals(checkbook.toArray(arrUnEx)));
    }

    @Test
    public void testToArrayOtherConstructorsV2() {
        Transaction[] arr = {t1};
        assertEquals(arr, checkbookWithDefault.toArray(arr));

        Transaction[] arr2 = {t1};
        assertEquals(arr2, checkbookWithOwner.toArray(arr2));
    }

    @Test
    public void testCheckBookValueTrue() {
        double exp = -450.0;
        assertEquals(exp, checkbook.checkbookValue());

        exp = 50.00;
        assertEquals(exp, checkbookWithOwner.checkbookValue());

        exp = 100.00;
        assertEquals(exp, checkbookWithDefault.checkbookValue());
    }

    @Test
    public void checkBookValueFalse() {
        double exp = 2.00;
        assertFalse(exp == checkbook.checkbookValue());
        assertFalse(exp == checkbookWithDefault.checkbookValue());
        assertFalse(exp == checkbookWithOwner.checkbookValue());

    }

    @Test
    public void checkBookValueAddRemove() {
        checkbook.add(t1);
        checkbookWithDefault.add(t1);
        checkbookWithOwner.add(t1);

        double expected = -950.00;
        assertEquals(expected, checkbook.checkbookValue());

        expected = -400.0;
        assertEquals(expected, checkbookWithDefault.checkbookValue());

        expected = -450.0;
        assertEquals(expected, checkbookWithOwner.checkbookValue());
    }

    @Test
    public void getTransactionAddingNewTForDateA() {
        LocalDate a = LocalDate.of(2018, 12, 05);
        Transaction t = new Transaction("foo", p1, TransactionType.DEBIT, 2.00);

        checkbookWithDefault.add(t);

        for (Transaction ta: checkbookWithDefault.getSpecificTransactions("date", a)) {
            assertEquals(a, ta.getDate());
        }
    }

    @Test
    public void getTransactionsForPayeeExists() {
        Payee payee = new Payee();
        Transaction t = new Transaction("test", payee, TransactionType.DEBIT, 10.00);
        Transaction[] expArr = {t};

        checkbook.add(t);
        checkbookWithDefault.add(t);
        checkbookWithOwner.add(t);

        assertEquals(expArr, checkbook.getSpecificTransactions("payee", payee));
        assertEquals(expArr, checkbookWithDefault.getSpecificTransactions("payee", payee));
        assertEquals(expArr, checkbookWithOwner.getSpecificTransactions("payee", payee));
    }

    // Contains all,
}